#include <usplash-theme.h>
#include <usplash_backend.h>

/**********************************************************************************
 Usplash theme to that is an ubuntu derivative of my MacX OSX looking boot theme.
 This theme will morph the gray apple into a colored ubuntu logo - keeping the
 spinner from the MacX theme.

 Thanks to all theme authors; especially usplash authors with custom animation for
 providing source as an example.
 
 Also, a very BIG thanks to whomever developed the logo.  I found this a while back
 and really liked it.  If you are the author, please email me and I will credit you.
 
 Graphics taken from the Mac4Lin project
 Visit them at http://sourceforge.net/projects/mac4lin
 
 Ubuntu logo taken from the web - if it is you, please let me know and I will update
 this to give you credit.
 
 NOTE:  I am not responsible if you fry your computer attempting to use this boot
        theme, use at your own risk.
 
 Changelog:
 -----------------------------------------------------------------------------------
 1.0 - initial coding
 
 ===================================================================================
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ==================================================================================
 
**********************************************************************************/

extern struct usplash_pixmap pixmap_bg_1024_768;
extern struct usplash_pixmap pixmap_bg_1280_1024;

extern struct usplash_pixmap pixmap_spinner;
extern struct usplash_font font_helvB10;

extern struct usplash_pixmap pixmap_morph_1;
extern struct usplash_pixmap pixmap_morph_2;
extern struct usplash_pixmap pixmap_morph_3;
extern struct usplash_pixmap pixmap_morph_4;
extern struct usplash_pixmap pixmap_morph_5;
extern struct usplash_pixmap pixmap_morph_6;
extern struct usplash_pixmap pixmap_morph_7;
extern struct usplash_pixmap pixmap_morph_8;
extern struct usplash_pixmap pixmap_morph_9;
extern struct usplash_pixmap pixmap_morph_10;
extern struct usplash_pixmap pixmap_morph_11;
extern struct usplash_pixmap pixmap_morph_12;
extern struct usplash_pixmap pixmap_morph_13;
extern struct usplash_pixmap pixmap_morph_14;
extern struct usplash_pixmap pixmap_morph_15;
extern struct usplash_pixmap pixmap_morph_16;
extern struct usplash_pixmap pixmap_morph_17;
extern struct usplash_pixmap pixmap_morph_18;
extern struct usplash_pixmap pixmap_morph_19;
extern struct usplash_pixmap pixmap_morph_20;
extern struct usplash_pixmap pixmap_morph_21;
extern struct usplash_pixmap pixmap_morph_22;

void t_init(struct usplash_theme* theme);
void t_clear_progressbar(struct usplash_theme* theme);
void t_draw_progressbar(struct usplash_theme* theme, int percentage);
void t_animate_step(struct usplash_theme* theme, int pulsating);
void spinner(struct usplash_theme* theme);
void morph(struct usplash_theme* theme, int percent);

struct usplash_theme usplash_theme;
struct usplash_theme usplash_theme_1280_1024;

static int spinner_x, spinner_y, spinner_part_width, spinner_height;
static int current_count = 0;
static int current_step = 0;
static int spinner_num_steps = 12;

// spinner_speed can be between 1 and 25
// there are 12 images in the spinner, so a value of 2 will make
// it spin around approx. once per second
static int spinner_speed = 2;

// array to hold all of morhing images
static struct usplash_pixmap morph_images[25];

/** ----------------------------------------------------------------------- **/

struct usplash_theme usplash_theme = {
	.version = THEME_VERSION,
	.next = &usplash_theme_1280_1024,
	.ratio = USPLASH_4_3,

	/* Background and font */
	.pixmap = &pixmap_bg_1024_768,
	.font   = &font_helvB10,

	/* Palette indexes */
	.background             = 0x01,
	.progressbar_background = 0x00,
	.progressbar_foreground = 0x1E,
	.text_background        = 0x01,
	.text_foreground        = 0x01,
	.text_success           = 0x23,
	.text_failure           = 0x24,

	/* Text box position and size in pixels */
	.text_x      			= 300,
	.text_y      			= 650,
	.text_width  			= 680,
	.text_height 			= 300,

	/* Text details */
	.line_height  			= 15,
	.line_length  			= 32,
	.status_width 			= 50,
	
	/* Progress bar position and size in pixels */
  	.progressbar_x      = 100,
  	.progressbar_y      = 100,
  	.progressbar_width  = 110,
  	.progressbar_height = 140,

	/* Functions */
	.init = t_init,
	.clear_progressbar = t_clear_progressbar,
	.draw_progressbar = t_draw_progressbar,
	.animate_step = t_animate_step,
};

struct usplash_theme usplash_theme_1280_1024 = {
	.version = THEME_VERSION,
	.next = NULL,
	.ratio = USPLASH_4_3,

	/* Background and font */
	.pixmap = &pixmap_bg_1280_1024,
	.font   = &font_helvB10,

	/* Palette indexes */
	.background             = 0x01,
	.progressbar_background = 0x00,
	.progressbar_foreground = 0x1E,
	.text_background        = 0x01,
	.text_foreground        = 0x01,
	.text_success           = 0x23,
	.text_failure           = 0x24,

	/* Text box position and size in pixels */
	.text_x      			= 300,
	.text_y      			= 650,
	.text_width  			= 680,
	.text_height 			= 300,

	/* Text details */
	.line_height  			= 15,
	.line_length  			= 32,
	.status_width 			= 50,
	
	/* Progress bar position and size in pixels */
  	.progressbar_x      = 100,
  	.progressbar_y      = 100,
  	.progressbar_width  = 110,
  	.progressbar_height = 140,

	/* Functions */
	.init = t_init,
	.clear_progressbar = t_clear_progressbar,
	.draw_progressbar = t_draw_progressbar,
	.animate_step = t_animate_step,
};

/** ---------------------------------------------------------- **/

/* init usplash */
void t_init(struct usplash_theme *theme) {
	// determine spinner position and dimensions
	spinner_height = pixmap_spinner.height;
	spinner_part_width = pixmap_spinner.width / spinner_num_steps;
	spinner_x = (theme->pixmap->width / 2) - (spinner_part_width / 2);
	spinner_y = (theme->pixmap->height / 2) + ((theme->pixmap->height / 2) / 2) - (pixmap_spinner.height / 2);
	
	// set text box dimensions and size
	theme->text_width = 600;
	theme->text_x = (theme->pixmap->width / 2) - (theme->text_width / 2);
	theme->text_y = 50;
	theme->text_height = 250;
	
	// set theme color indexes
	theme->background             = 0;
	theme->progressbar_background = 0;
	theme->progressbar_foreground = 0;
	theme->text_background        = 0;
	theme->text_foreground        = 160;
	theme->text_success           = 190;
	theme->text_failure           = 255;
	
	// set progress bar specs
	theme->progressbar_x = (theme->pixmap->width / 2) - (pixmap_morph_1.width / 2);
	theme->progressbar_y = (theme->pixmap->height / 2) - (pixmap_morph_1.height / 2);
	theme->progressbar_width = pixmap_morph_1.width;
	theme->progressbar_height = pixmap_morph_1.height;
	
	// load morph array
	morph_images[0] = pixmap_morph_1;
	morph_images[1] = pixmap_morph_2;
	morph_images[2] = pixmap_morph_3;
	morph_images[3] = pixmap_morph_4;
	morph_images[4] = pixmap_morph_5;
	morph_images[5] = pixmap_morph_6;
	morph_images[6] = pixmap_morph_7;
	morph_images[7] = pixmap_morph_8;
	morph_images[8] = pixmap_morph_9;
	morph_images[9] = pixmap_morph_10;
	morph_images[10] = pixmap_morph_11;
	morph_images[11] = pixmap_morph_12;
	morph_images[12] = pixmap_morph_13;
	morph_images[13] = pixmap_morph_14;
	morph_images[14] = pixmap_morph_15;
	morph_images[15] = pixmap_morph_16;
	morph_images[16] = pixmap_morph_17;
	morph_images[17] = pixmap_morph_18;
	morph_images[18] = pixmap_morph_19;
	morph_images[19] = pixmap_morph_20;
	morph_images[20] = pixmap_morph_21;
	morph_images[21] = pixmap_morph_22;
	morph_images[22] = pixmap_morph_22;
	morph_images[23] = pixmap_morph_22;
	morph_images[24] = pixmap_morph_22;
	morph_images[25] = pixmap_morph_22;
}

/******
 * Animation callback - called 25 times per second by Usplash 
 * 
 * Param:	struct usplash_theme* theme	- theme being used 
 * 			int pulsating - boolean int
 */
void t_animate_step(struct usplash_theme* theme, int pulsating) {
	current_count = current_count + 1;
	static int morph_count = 0;
	static int morph_pause = 1;
	
	// call morph animation
	if(morph_pause && morph_count > 25) {
		morph_pause = 0;
		morph_count = 0;
	}
	
	if(! morph_pause) {
		morph(theme, morph_count);
	}
	
	morph_count++;
	
	// increase test int for slower spinning
	if(current_count == spinner_speed) {
		spinner(theme);
		current_count = 0;	
	}
}

/********
 * Animate the spinner
 *  helper function to aid in animation of spinner
 */
void spinner(struct usplash_theme* theme) {
	current_step = current_step + 1;
	
	int x0 = (spinner_part_width * current_step) - spinner_part_width;
	int y0 = 0;
	
	// if current step > number of images in the spinner, then reset to beginning (at end or circular spinner)
	if(current_step >= spinner_num_steps) {
		current_step = 0;
	}
	
	// call usplash_put_part for the spinner image
	usplash_put_part(spinner_x, spinner_y, spinner_part_width, spinner_height, &pixmap_spinner, x0, y0);
}

/********
 * Animate the morph
 *  helper function to aid in animation of morphing images
 */
void morph(struct usplash_theme* theme, int percent) {
	int percent_offset = 2;
	if(percent > 100) {
		percent = 100;
	}
	
	int index = (percent / percent_offset) / 2;
	if(index < 0) {
		index = index * -1;
	}
	
	if(index > 25) {
		index = 25;
	}
	
	usplash_put_part(theme->progressbar_x, theme->progressbar_y, morph_images[index].width, morph_images[index].height, &morph_images[index], 0, 0);
}

/** not used for now **/
void t_clear_progressbar(struct usplash_theme *theme) { 
	usplash_put_part(theme->progressbar_x, theme->progressbar_y, pixmap_morph_1.width, pixmap_morph_1.height, &pixmap_morph_1, 0, 0);
}

/** animation for progress of boot **/
void t_draw_progressbar(struct usplash_theme *theme, int percentage) {
	if(percentage != 0) {
		//morph(theme, percentage);
	}
	
}
